# -*- coding: cp1252 -*-

import os, sys
from os.path import join, isdir
from ConfigParser import ConfigParser

# name of the property file
propFile = "build.properties"


class MyConfigParser(ConfigParser):
    ''' Custom ConfigParser with no "to-lower-case"'''
    
    def optionxform (self, optionstr):
        return optionstr



# mandatory options from build.properties
mcu=""
freq=""
target=""
SRCDIRS=""

# optional options from build.properties
buildOptions = {}

# internal variables
rootdirs = []       # root source directories 
srcdirs = []        # source directories (full path)
subdirsGCC = []     # gcc subdirs (without VPATH part)
subdirs = []        # subdirs (full path)
csrcfiles = []      # list of cv files
cppsrcfiles = []    # list of cpp files 

	
def readProperties():
    cfg = MyConfigParser()
    cfg.read(propFile)
    
    globals()["mcu"] = cfg.get("config", "MCU")
    globals()["freq"] = cfg.get("config", "F_CPU")
    globals()["target"]= cfg.get("config", "TARGET")
    
    srcTemp = ".,"
    srcTemp = srcTemp + cfg.get("config", "EXTSRCDIRS")
    temp = srcTemp.split(',')
    for dir in temp:
        # make shure we have a trailing /
        if not dir.endswith("/"):
            dir = dir + "/"
        rootdirs.append(dir)
    
    for n, v in cfg.items("options"):
        lis = v.split(",")
        globals()["buildOptions"][n] = lis;
    

def findSrcSubdirs(srcdir):
    ''' Find all subdirs of the sourcedir provided '''
    items  = os.listdir(srcdir)
    for item in items:
            if isdir(join(srcdir, item)) and not item[0:1] == ".":
                findSrcSubdirs(srcdir + "/" + item)
                subdirs.append(srcdir + "/" + item)

def findCppFiles():
        for dir in subdirs:
                items = os.listdir(dir)                
                for item in items:
                        if (not isdir(item)) and item.endswith(".cpp"):
                                cppsrcfiles.append(dir + "/" + item)

def findCFiles():
        for dir in subdirs:
                items = os.listdir(dir)                
                for item in items:
                        if (not isdir(item)) and item.endswith(".c"):
                                csrcfiles.append(dir + "/" + item)

def build():
        ''' Build the project '''


        
        # strip root dirs from source file names and directories
        subdirsGCC = stripRootDirs(subdirs)
        subdirsGCC.sort()   
        SUBDIRS =  '"' + " ".join(subdirsGCC) + '"'

        csrcfilesGCC = stripRootDirs(csrcfiles)
        csrcfilesGCC.sort()
        CSRC = '"' + ' '.join(csrcfilesGCC)+ '"'
        
        cppsrcfilesGCC = stripRootDirs(cppsrcfiles)
        cppsrcfilesGCC.sort()
        CPPSRC = '"' + ' '.join(cppsrcfilesGCC)+ '"'
        
        # touch some variables


        # build the command
        command = "make " + " MCU=" + mcu + " F_CPU=" + freq + \
            " TARGET=" + target + \
            " SUBDIRS=" + SUBDIRS + \
            " CPPSRC=" + CPPSRC + \
            " SRC=" + CSRC 
            
        for optKey in buildOptions.keys():
            vals = buildOptions[optKey]
            command = command + " " + optKey + "=" + '"' + " ".join(vals) + '"'
            

        # execute the command
        print command
        sts = os.system(command)
        
        

def stripRootDirs(filelist):
    ret = []
    for file in filelist:
        f = stripRootDir(file)
        if f not in ret:
            ret.append(f)
    return ret
    
def stripRootDir(dir):
    tmp = dir
    for rdir in rootdirs:
        if dir.startswith(rdir):
            tmp = dir.lstrip(rdir)
            break
            
    if tmp.startswith("/"):
        tmp.lstrip("/")
    if tmp.startswith("\\"): 
        tmp.lstrip("\\")
    return tmp
        

# ######################################
# ## Main build script
# ######################################

# read all config properties
readProperties()

# find all source directories recursively
for dir in rootdirs:
    findSrcSubdirs(dir)

           
# find .c and .cpp files
findCFiles()
findCppFiles()

# start the build
build()

print mcu, freq
        





